@@ -3,8 +3,8 @@ package ExtUtils::CBuilder::Base;
 use strict;
 use File::Spec;
 use File::Basename;
-use IO::File;
 use Config;
+use Text::ParseWords;
 
 use vars qw($VERSION);
 $VERSION = '0.00_01';
@@ -13,12 +13,28 @@ sub new {
   my $class = shift;
   my $self = bless {@_}, $class;
 
+  $self->{properties}{perl} = $class->find_perl_interpreter
+    or warn "Warning: Can't locate your perl binary";
+
   while (my ($k,$v) = each %Config) {
     $self->{config}{$k} = $v unless exists $self->{config}{$k};
   }
   return $self;
 }
 
+sub find_perl_interpreter {
+  my $perl;
+  File::Spec->file_name_is_absolute($perl = $^X)
+    or -f ($perl = $Config::Config{perlpath})
+    or ($perl = $^X);
+  return $perl;
+}
+
+sub add_to_cleanup {
+  my $self = shift;
+  my %files = map {$_, 1} @_;
+}
+
 sub object_file {
   my ($self, $filename) = @_;
 
@@ -27,49 +43,64 @@ sub object_file {
   return "$file_base$self->{config}{obj_ext}";
 }
 
-sub compile_library {
+sub arg_include_dirs {
+  my $self = shift;
+  return map {"-I$_"} @_;
+}
+
+sub arg_nolink { '-c' }
+
+sub arg_object_file {
+  my ($self, $file) = @_;
+  return ('-o', $file);
+}
+
+sub compile {
   my ($self, %args) = @_;
-  die "Missing 'source' argument to compile_library()" unless defined $args{source};
+  die "Missing 'source' argument to compile()" unless defined $args{source};
   
   my $cf = $self->{config}; # For convenience
 
   $args{object_file} ||= $self->object_file($args{source});
   
-  my @include_dirs = map {"-I$_"} (@{$args{include_dirs} || []},
-				   File::Spec->catdir($cf->{installarchlib}, 'CORE'));
+  my @include_dirs = $self->arg_include_dirs
+    (@{$args{include_dirs} || []},
+     File::Spec->catdir($cf->{installarchlib}, 'CORE'));
   
   my @extra_compiler_flags = $self->split_like_shell($args{extra_compiler_flags});
   my @cccdlflags = $self->split_like_shell($cf->{cccdlflags});
   my @ccflags = $self->split_like_shell($cf->{ccflags});
   my @optimize = $self->split_like_shell($cf->{optimize});
-  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags, '-c', @ccflags, @optimize);
+  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags,
+	       $self->arg_nolink,
+	       @ccflags, @optimize,
+	       $self->arg_object_file($args{object_file}),
+	      );
   
   my @cc = $self->split_like_shell($cf->{cc});
   
-  $self->do_system(@cc, @flags, '-o', $args{object_file}, $args{source})
+  $self->do_system(@cc, @flags, $args{source})
     or die "error building $args{object_file} from '$args{source}'";
 
   return $args{object_file};
 }
 
-sub compile_executable {
-  die "Not implemented yet";
-}
-
-sub have_c_compiler {
+sub have_compiler {
   my ($self) = @_;
   return $self->{have_compiler} if defined $self->{have_compiler};
   
   my $tmpfile = File::Spec->catfile(File::Spec->tmpdir, 'compilet.c');
   {
-    my $fh = IO::File->new("> $tmpfile") or die "Can't create $tmpfile: $!";
-    print $fh "int boot_compilet() { return 1; }\n";
+    local *FH;
+    open FH, "> $tmpfile" or die "Can't create $tmpfile: $!";
+    print FH "int boot_compilet() { return 1; }\n";
+    close FH;
   }
 
   my ($obj_file, @lib_files);
   eval {
-    $obj_file = $self->compile_library(source => $tmpfile);
-    @lib_files = $self->link_objects(objects => $obj_file, module_name => 'compilet');
+    $obj_file = $self->compile(source => $tmpfile);
+    @lib_files = $self->link(objects => $obj_file, module_name => 'compilet');
   };
   warn $@ if $@;
   my $result = $self->{have_compiler} = $@ ? 0 : 1;
@@ -87,9 +118,17 @@ sub lib_file {
   return "$dl_file.$self->{config}{dlext}";
 }
 
-sub need_prelink_objects { 0 }
 
-sub prelink_objects {
+sub exe_file {
+  my ($self, $dl_file) = @_;
+  $dl_file =~ s/\.[^.]+$//;
+  $dl_file =~ tr/"//d;
+  return "$dl_file$self->{config}{_exe}";
+}
+
+sub need_prelink { 0 }
+
+sub prelink {
   my ($self, %args) = @_;
   
   ($args{dl_file} = $args{dl_name}) =~ s/.*::// unless $args{dl_file};
@@ -109,28 +148,41 @@ sub prelink_objects {
   return grep -e, map "$args{dl_file}.$_", qw(ext def opt);
 }
 
-sub link_objects {
+sub link {
   my ($self, %args) = @_;
+  return $self->_do_link('lib_file', lddl => 1, %args);
+}
+
+sub link_executable {
+  my ($self, %args) = @_;
+  return $self->_do_link('exe_file', lddl => 0, %args);
+}
+				   
+sub _do_link {
+  my ($self, $type, %args) = @_;
+
   my $cf = $self->{config}; # For convenience
   
   my $objects = delete $args{objects};
   $objects = [$objects] unless ref $objects;
-  $args{lib_file} ||= $self->lib_file($objects->[0]);
+  my $out = $args{$type} || $self->$type($objects->[0]);
   
-  my @temp_files = 
-    $self->prelink_objects(%args,
-			   dl_name => $args{module_name}) if $self->need_prelink_objects;
+  my @temp_files;
+  @temp_files =
+    $self->prelink(%args,
+		   dl_name => $args{module_name}) if $self->need_prelink;
   
   my @linker_flags = $self->split_like_shell($args{extra_linker_flags});
-  my @lddlflags = $self->split_like_shell($cf->{lddlflags});
+  my @lddlflags = $args{lddl} ? $self->split_like_shell($cf->{lddlflags}) : ();
   my @shrp = $self->split_like_shell($cf->{shrpenv});
   my @ld = $self->split_like_shell($cf->{ld});
-  $self->do_system(@shrp, @ld, @lddlflags, '-o', $args{lib_file}, @$objects, @linker_flags)
-    or die "error building $args{lib_file} from @$objects";
+  $self->do_system(@shrp, @ld, @lddlflags, '-o', $out, @$objects, @linker_flags)
+    or die "error building $out from @$objects";
   
-  return wantarray ? ($args{lib_file}, @temp_files) : $args{lib_file};
+  return wantarray ? ($out, @temp_files) : $out;
 }
 
+
 sub do_system {
   my ($self, @cmd) = @_;
   print "@cmd\n";
@@ -140,15 +192,12 @@ sub do_system {
 sub split_like_shell {
   my ($self, $string) = @_;
   
-  return () unless defined($string) && length($string);
+  return () unless defined($string);
   return @$string if UNIVERSAL::isa($string, 'ARRAY');
+  $string =~ s/^\s+|\s+$//g;
+  return () unless length($string);
   
-  return $self->shell_split($string);
+  return Text::ParseWords::shellwords($string);
 }
 
-sub shell_split {
-  return split ' ', $_[1];  # XXX This is naive - needs a fix
-}
-
-
 1;
